/* The ATmega32 USART (Universal synchronous and asynchronous receiver and
   transmitter) is discussed starting on page 140 of the ATmega32 data sheet. */

/* In order to receive 8-bit characters on your Linux computer, do something
   like this:
   	stty -F /dev/ttyS0 raw 38400
*/

#include <avr/io.h>

#include "defs.h"

#define SERIAL_BAUD_RATE 38400UL
/*
#define SERIAL_BAUD_RATE 115200UL
*/
/* See the ATmega32 data sheet, page 143 for the source of this formula. */
#define UBRR_BAUD (F_CPU / (16 * SERIAL_BAUD_RATE) - 1)

#if defined (__AVR_ATmega128__)
#define UBRRH UBRR0H
#define UBRRL UBRR0L
#define UCSRA UCSR0A
#define UCSRB UCSR0B
#define UDR UDR0
#endif

/* For USART initialization, see the ATmega32 data sheet, page 146. */
void serial_init(void)
{
	/* Set the baud rate. The UBRRH register shares the same location as the
	   UCSRC register. Bit 7 (URSEL) must be zero to write to UBRRH. See the
	   ATmega32 data sheet, page 158. */
	UBRRH = (unsigned char) (UBRR_BAUD >> 8);
	UBRRL = (unsigned char) UBRR_BAUD;
	/* Set the TXEN (enable transmit) bit on the USART. */
	UCSRB |= _BV(TXEN);
	/* The UCSRC register controls synchronous/asynchronous mode, parity and
	   stop bits, etc. The defaults are fine. */
}

void serial_putchar(char c)
{
	/* Wait until the UDRE (USART Data Register Empty) bit is set. See the
	   ATmega32 data sheet, page 161. */
	loop_until_bit_is_set(UCSRA, UDRE);
	UDR = c;
}

void serial_write(const unsigned char *buffer, unsigned int size)
{
	for ( ; size > 0; size--)
		serial_putchar(*buffer++);
}

void serial_puts(const char *s)
{
	for ( ; *s != '\0'; s++)
		serial_putchar(*s);
}
