#include <avr/io.h>

#include "defs.h"
#include "servo.h"

void servo_init(void)
{
	/* Set the servo control pin to output. */
	SERVO_DDR |= _BV(SERVO);
	/* Turn on timer/counter 0 with a prescaler of 1024. See the ATmega32
	   data sheet, page 82. */
	TCCR0 |= _BV(CS02) | _BV(CS00);
	/* Set the waveform generation mode bits (WGM01 and WGM00) to fast PWM
	   mode. See the ATmega32 data sheet, page 80. */
	TCCR0 |= _BV(WGM01) | _BV(WGM00);
	/* Set the output pin (OC0) to clear on compare match and set on TOP.
	   See the ATmega32 data sheet, page 81. */
	TCCR0 |= _BV(COM01);
}

void servo_set_position(enum servo_position pos)
{
	/* The servo works by receiving a pulse 50 times a second, or more
	   frequently. A pulse of 1 ms moves the servo all the way
	   counterclockwise, and a pulse of 2 ms moves the servo all the way
	   clockwise. With a prescaler of 1024, the frequency of the PWM channel
	   is F_CPU / (1024 * 256) (see the ATmega32 data sheet, page 76). Then
	   the period is 1024 * 256 / F_CPU. So with F_CPU = 16000000 (16 MHz),
	   a counterclockwise pulse should be
	   0.001 / (1024 * 256 / F_CPU) * 256 = 15.625 (rounds to 16)
	   And a clockwise pulse should be
	   0.002 / (1024 * 256 / F_CPU) * 256 = 31.250 (rounds to 31).

	   But in actual practice, these values don't cover the whole range.
	   These values are based on trial and error to find the approximate
	   limits. */
	if (pos == SERVO_FULL)
		OCR0 = 36;
	else if (pos == SERVO_HALF)
		OCR0 = 22;
	else
		OCR0 = 8;
}
